#include <stdio.h>
#include <unistd.h>
#include <netdb.h>
#include <ctype.h>
#include <string.h>
#include <netinet/in.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <stdarg.h>
#include "config.h"
#include "both.h"

#ifdef TIMEOUT
#include <sys/time.h>
#include <sys/types.h>
#include <errno.h>
#endif

/*-----------------------------------------------------*/
/* get next number in string */
char *number(char *sp, int *intPtr) {
	int i,c;

	if(sp==NULL) {
		*intPtr=0; 
		return(sp);
	}

	for (i=0;sp[i]!=' ';i++) {
		if(sp[i]=='\0') {
			break;
		}
	}
	c=sp[i];
	sp[i]='\0';
	sscanf(sp,"%d",intPtr);
	sp[i]=c;
	/* if at EOS return the NULL, else skip space */
	return (sp[i] == '\0') ? sp+i : sp+(++i);
}
/*---------------------------------------------*/
struct hostent *get_hostent(char *host) {
	struct in_addr saddr;
	int c;
	struct hostent *hi = NULL;

	if(host==NULL) { 
		fprintf(stderr,"No hostname given !\r\n"); 
	}
	else {
		c=*host;
		if(isdigit(c)) {
 			saddr.s_addr = inet_addr(host); 
 			hi = gethostbyaddr((char *)&saddr,sizeof(struct in_addr),AF_INET);
		}
		else {
			hi = gethostbyname(host);
		}
	}
	return hi;
}
/*--------------------------------------------*/
int connect_to_nntphost(char *host, struct hostent **hi) {
	char *ptr;
	struct in_addr *aptr;
	struct in_addr saddr;
	struct sockaddr_in address;
	int sockfd = -1;

	/* Find the internet address of the NNTP server */
 	*hi = get_hostent(host);
 	if(*hi == NULL) {
		fprintf(stderr,"%s :",host);
  		perror("Could not get host information:");
 	}
	else {
		printf("Official host name: %s\r\n", (*hi)->h_name );
 		while((ptr = *((*hi)->h_aliases)) != NULL) {
			printf("Alias %s\r\n", ptr );
			(*hi)->h_aliases++;
		}
 		if((*hi)->h_addrtype != AF_INET) {
  			fprintf(stderr,"Unsupported address type.\r\n" );
		}
		else {
			while((aptr = (struct in_addr *)*((*hi)->h_addr_list)++) != NULL) {
				saddr = *aptr;
				printf("Address: %s\r\n", inet_ntoa(*aptr));
			}

			/* Create a socket */
 			if((sockfd = socket( AF_INET, SOCK_STREAM, SOCKET_PROTOCOL)) == -1) {
				perror("Socket failed:");
			}
			else { 
				address.sin_family = AF_INET;
				address.sin_port = htons(119);  /* NNTP port */
				address.sin_addr= saddr;

				/* Establish a connection */
				if(connect(sockfd, (struct sockaddr *)&address, sizeof address ) == -1) {
					perror("Connect failed:");
					close(sockfd);
					sockfd = -1;
				}
				else {
					printf("Connected to %s\r\n",(*hi)->h_name);
				}
			}
		}
	}
	return sockfd;
}
/*----------------------------------------------------------------*/
int sputline(int fd, char *outbuf) {

#ifdef DEBUG1
	do_debug("\nSENT: %s", outbuf);
#endif
	return send(fd, outbuf, strlen(outbuf), 0);
}
/*-------------------------------------------------------------*/
#ifdef DEBUG
void do_debug(char *fmt, ...) {

	FILE *fptr = NULL;
	va_list args;

	if((fptr = fopen(N_DEBUG, "a")) == NULL) {
		fptr = stderr;
	}
		
	va_start(args, fmt);
	vfprintf(fptr, fmt, args);
	va_end(args);

	if(fptr != stderr) {
		fclose(fptr);
	}
}
/*------------------------------------------------------------*/
void MyPerror(char *message) {
	
	perror(message);
	do_debug("%s\n", message);
}	
#endif
/*-----------------------------------------------------------*/
int sgetline(int fd, char **inbuf) {

	static char buf[MAXLINLEN+MAXLINLEN+6];
	static char *start = buf;
	static char *eob = buf;		/* end of buffer */
	int ret, i, len;
	char *ptr;
	fd_set myset;
	struct timeval mytimes;

	ret = 0;

	if((ptr = strstr(start,"\r\n")) == NULL) {
		/* not a full line in buffer */
		i = eob-start; 	/* length of partial line in buf */
		if((eob - buf) > MAXLINLEN) {
#ifdef DO_DEBUG1
			do_debug("SHIFTING BUFFER\n");
#endif
			/* not enuf room in buffer for a full recv */
			bcopy(start, buf, i);		/* move to start of buf */
			eob = buf + i;
			*eob = '\0';
			start = buf;			/* reset pointers */
		}
		len = i;
		/* try to get a line in, up to maxlen */
		do {
#ifdef TIMEOUT
			/* handle timeout value */
			FD_ZERO(&myset);
			FD_SET(fd, &myset);
			mytimes.tv_sec = TIMEOUT;
			mytimes.tv_usec = 0;
	
			if((i = select(FD_SETSIZE, &myset, (fd_set *) NULL, (fd_set *) NULL, &mytimes)) == 1) {
				i = recv(fd, eob, MAXLINLEN-len, 0);	/* get line */
			}
			else {
				/* error */
				if(i == 0) {
					/* no bytes available */
					errno =  ETIME;
				}
				i = -1;
			}
#else 
			i = recv(fd, eob, MAXLINLEN-len, 0);	/* get line */
#endif
			if(i < 1) {
				perror("Socket error");
				ret = -1;
			}
			else {
				eob += i;	/* increment buffer end */
				*eob = '\0';	/* NULL terminate it  */
#ifdef DEBUG1
				do_debug("\nGOT: %.*s",i,eob-i);
#endif 
				len += i;				
				ptr = strstr(start, "\r\n");		
			}
		} while(ptr == NULL && len < MAXLINLEN && ret == 0);
	}
	if(ptr != NULL) {
		/* we have a full line left in buffer */
		*ptr++ = '\n';	/* change \r\n to just \n */
		*ptr++ = '\0';	/* null terminate */
		*inbuf = start;
		ret = (ptr-1) - start;	/* length of string */
		start = ptr; 	/* skip \r\n */
		if(eob == start) {
			/* nothing left in buffer, reset pointers to start of buffer */
			/* to give us a full buffer to receive next data into */
			/* hopefully doing this will mean less buffer shifting */
			/* meaning faster receives */
			eob = start = buf;
#ifdef DEBUG1
			do_debug("Empty buffer, resetting to start\n");
#endif
		}
	}
	else if(ret == 0) {
		/* partial line in buffer */
		/* null terminate */
		*eob = '\0';
		*inbuf = start;
		ret = (eob-1) - start;	/* length of string */
		start = ++eob;	/* point both past end */
	}
#ifdef DEBUG1
	if(ret > 0) {
		do_debug("\nRETURNING len %d: %s", ret, *inbuf);
	}
#endif  
	return ret;
}		
/*----------------------------------------------------*/
