#include <stdio.h>
#include <stdlib.h>
#include <netdb.h>
#include <unistd.h>
#include <string.h>

#include "config.h"
#include "both.h"
#include "suck.h"
#include "killfile.h"
#ifdef TIMER
#include "timer.h"
#endif

/* local function prototypes */
char *get_a_chunk_mem(int);
int chk_msg_kill(PKillStruct, char *);


/*-------------------------------------------------------------------------*/
const char *Reasons[] = {
	"No Reason - major screwup",
	"Number of lines exceeded"
};
enum { REASON_NONE, REASON_TOOMANYLINES };

/*--------------------------------------------------------------------------*/
PKillStruct parse_killfile() {
	static KillStruct mykill;
	PKillStruct retptr;
	FILE *fptr;
	char buf[MAXLINLEN+1];

	/* first initialize it */
	mykill.msglines = 0;
	retptr = &mykill;
	
	/* now read in the killfile and parse it */
	if((fptr = fopen(full_path(FP_GET, FP_DATADIR, N_KILLFILE), "r")) == NULL) {
		retptr = NULL;
	}
	else {
		while(fgets(buf, MAXLINLEN, fptr) != NULL) {
#ifdef DEBUG2
			do_debug("Read kill file line:%s\n", buf);
#endif
			if(strncmp(buf, "LINES=", 6) == 0) {
				sscanf(&buf[6], "%d", &(mykill.msglines));
#ifdef DEBUG2
				do_debug("Killfile msglines = %d\n", mykill.msglines);
#endif
			}
		}
		fclose(fptr);
	}

	return retptr;
}	
/*--------------------------------------------------------------------------*/
int get_one_article_kill(int sockfd, char *msgnr, int nron, int articleCount, int logcount, int MultiFile, PKillStruct killp) {

	char buf[MAXLINLEN+1], *inbuf;
	int retval, response;
	FILE *fptr;

	retval = RETVAL_OK;

	/* build command to get article header*/
	sprintf(buf, "head %s\r\n", msgnr);
	response = send_a_command(sockfd, buf, 221);

	if(response == RETVAL_ERROR) {
		retval = RETVAL_ERROR;
	}
	else if(response == RETVAL_OK) {

		if((inbuf = get_a_chunk_mem(sockfd)) == NULL) {
			retval = RETVAL_ERROR;
		}
		else if(chk_msg_kill(killp, inbuf)==FALSE){
			if(MultiFile == TRUE) {
				/* open file */
				/* file name will be ####-#### ex 001-166 (nron,total) */
				sprintf(buf,"%0*d-%d", logcount, nron ,articleCount);
				if((fptr = fopen(full_path(FP_GET, FP_MSGDIR, buf), "w")) == NULL) {
					perror(full_path(FP_GET, FP_MSGDIR,buf));
					retval = RETVAL_ERROR;
				}
				else {
					fputs(inbuf, fptr);
					fputs("\n", fptr);	/* needed */
					sprintf(buf, "body %s\r\n", msgnr);
					response = send_a_command(sockfd, buf, 222);
					switch(response) {
					  case RETVAL_OK:
						retval = get_a_chunk(sockfd, fptr);
						fclose(fptr);
						break;
					  case RETVAL_ERROR:
						retval = RETVAL_ERROR;
						/* fall thru */
					  case RETVAL_UNEXPECTEDANS:
						fclose(fptr);
						sprintf(buf,"%0*d-%d", logcount, nron ,articleCount);
						unlink(buf);
					}
				}
			}
			else {
				fputs(inbuf, stdout);
				fputs("\n", stdout);
				sprintf(buf, "body %s\r\n", msgnr);
				response = send_a_command(sockfd, buf, 222);
				switch(response) {
				  case RETVAL_OK:
					retval = get_a_chunk(sockfd, stdout);
					/* this is needed as a separator */
					/* in stdout version */
					fputs(".\n", stdout);
				  case RETVAL_ERROR:
					retval = RETVAL_ERROR;
					break;
				  case RETVAL_UNEXPECTEDANS:
					/* nothing to do  */
					break;
				}			
			
			}
		}
	}
	return retval;
}
/*---------------------------------------------------------------*/
/* this routine is same as get_a_chunk() except it goes to memory */
char *get_a_chunk_mem(int sockfd) {

	static char *buf = NULL;
	static int bufsize = 8192;

	int retval, done, partial, len, currbuf;
	char *inbuf, *newbuf;
	
	retval = RETVAL_OK;
	done = FALSE;
	partial = FALSE;
	currbuf = 0;
	if(buf == NULL) {
#ifdef DEBUG2
		do_debug("allocing memory, size = %d\n", bufsize);
#endif
		if((buf=malloc((size_t) bufsize)) == NULL) {
			fprintf(stderr, "Out of Memory");
		}
	}
	while(buf != NULL && done == FALSE) {
		len=sgetline(sockfd, &inbuf);
#ifdef TIMER
		TimerFunc(TIMER_ADDBYTES, len);
#endif
		if(len < 0) {
			free(buf);
			buf = NULL;
		}
		else if(partial == FALSE && len == 2 && strcmp(inbuf, ".\n") == 0) {
			done = TRUE;
		}
		else {
			if((len+currbuf) > bufsize) {
				/* buffer not big enough realloc */
				bufsize = len+currbuf;
				if((newbuf = realloc(buf, (size_t) bufsize)) == NULL) {
					free(buf);
					buf = NULL;
					fprintf(stderr, "Out of Memory");
#ifdef DEBUG2
					do_debug("Re-alloc failed\n");
#endif
				}
				else {
					buf = newbuf;
#ifdef DEBUG2
					do_debug("Re-alloc succeeded, new size = %d\n", bufsize);
#endif
				}
			}	
			if(buf != NULL) {
				strcpy(buf+currbuf, inbuf);
				currbuf += len;
				partial= (len==MAXLINLEN&&inbuf[len-1]!='\n') ? TRUE : FALSE;
			}
		}
	}
	return buf;
}
/*-------------------------------------------------------------------------*/
/* chk_msg_kill - return TRUE if kill msg, FALSE if keep                   */
/* if kill msg, add it to killlog 					   */
int chk_msg_kill(PKillStruct killp, char *headerbuf) {

	char *startline;
	int nrlines, why, killyn;
	FILE *fptr;

	killyn = FALSE;
	why = REASON_NONE;

	/* check killlines first */
	if(killp->msglines > 0) { 
		if((startline = strstr(headerbuf, "Lines: ")) != NULL)  {
			nrlines = 0;	/* just in case */
			sscanf(startline+6, "%d", &nrlines);
#ifdef DEBUG2
			do_debug("Msg has %d lines, killlines = %d", nrlines, killp->msglines);
#endif
			if(killp->msglines < nrlines) {
				/* JACKPOT */
				killyn = TRUE;
				why = REASON_TOOMANYLINES;
			}
		}
		
	}
	if(killyn == TRUE) {
		/* log it */
		if((fptr = fopen(full_path(FP_GET, FP_TMPDIR, N_KILLLOG), "a")) == NULL) {
			perror("Unable to log killed msg");
		}
		else {
			fprintf(fptr, "MESSAGE KILLED - reason = %s\n%s\n", Reasons[why], headerbuf);
			fclose(fptr);
		}
	}
	return killyn;			
}
