#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <netdb.h>
#include <dirent.h>
#include <string.h>
#include <sys/stat.h>
#include <errno.h>
#include <limits.h>
#include <math.h>
#include "config.h"
#include "both.h"
#include "suck.h"

#ifdef KILLFILE
#include "killfile.h"
#endif

#ifdef TIMER
#include "timer.h"
#endif

/* for use in do_dedupe build_list and allocnode */
typedef struct LinkList {
	struct LinkList *next;
	char *msgnr;
	char mandatory;
} List, *PList;

/* function prototypes */
int get_message_index(int *, int);
int get_articles(long, int, int, int, int, int);
int get_one_article(int, char *, int, int, int, int);
int get_announcements(int);
int do_dedupe(void);
int checkdir(char *);
int do_innbatch(int, char *);
int do_rnewsbatch(int, char *);
PList allocnode(char *, int);
int build_list(char *, PList *, int *, int *, int);

enum { BATCH_FALSE, BATCH_INNXMIT, BATCH_RNEWS };	/* poss values for batch variable */
enum { MANDATORY_YES = 'M' , MANDATORY_OPTIONAL = 'O' };			/* do we have to download this message */
enum { STATUS_STDOUT, STATUS_STDERR };
/*------------------------------------------------------------------*/
void main(int argc, char *argv[]) {
	
	FILE *fp;
	int sockfd;
	struct hostent *hi;

	char *host = NULL, *batchfile = NULL, *ptr;
	long offset=0;
	int loop, batch = BATCH_FALSE, nron = 0, articleCount=0, retval = RETVAL_OK, MultiFile = FALSE,
		status = STATUS_STDOUT;

	/* allow no args, only the hostname, or hostname as first arg */
	host = getenv("NNTPSERVER");		/* the default */
	switch(argc) {
	  case 1:
		break;
	  case 2:   
		host = argv[1];
		break;
	  default:
		loop = 1;
		if(argv[1][0] != '-') {
			host = 	argv[loop++];
		}			
		for(;loop<argc;loop++) {
			if(argv[loop][0] == '-') {
				switch(argv[loop][1]) {
				  case 's':	/* force the status report to stdout */
					status = STATUS_STDERR;
					break;
				  case 'h':
					host = (loop+1 == argc) ? getenv("NNTPSERVER") : argv[++loop];
					break;	
				  case 'm':
					MultiFile = TRUE;
					break;
				  case 'd':	/* change one of the directory names */
					if(loop+1 == argc) { 
						fprintf(stderr, "No directory name provided\n");
						retval = RETVAL_ERROR;
					}
					else {
						ptr = NULL;
						switch (argv[loop][2]) {
						  case 't':
							ptr = full_path(FP_SET, FP_TMPDIR, argv[++loop]);
							break;
						  case 'd':
							ptr = full_path(FP_SET, FP_DATADIR, argv[++loop]);
							break;
						  case 'm':
							ptr = full_path(FP_SET, FP_MSGDIR, argv[++loop]);
							break;
						}
						if(ptr == NULL) { 
							fprintf(stderr, "Invalid directory\n");
							retval = RETVAL_ERROR;
							break;
						}
					}
					break;		 
				  case 'b':	/* batch file implies MultiFile Mode */
					switch (argv[loop][2]) {
					  case 'i':
						batch = BATCH_INNXMIT;
						MultiFile = TRUE;		
						break;
					  case 'r':
						batch = BATCH_RNEWS;
						MultiFile = TRUE;
						break;
					  default:
						fprintf(stderr, "Invalid Batch arg: %s\n", argv[loop]);
						retval = RETVAL_ERROR;
						break;
				  	}
					if(retval == RETVAL_OK && loop+1 == argc) {
						fprintf(stderr, "No Batch file name provided\n");
						retval = RETVAL_ERROR;
					}
					else {
						batchfile = argv[++loop];
					}
					break;
				  default:
					fprintf(stderr, "Invalid argument: %s\n", argv[loop]);
					retval = RETVAL_ERROR;
					break;
				}			
			}
			else {
				fprintf(stderr, "Invalid argument: %s\n", argv[loop]);
				retval = RETVAL_ERROR;
			}	
		}
		break;
	}
	if(retval == RETVAL_OK) {
		sockfd = connect_to_nntphost( host, &hi);
		if(sockfd < 0 ) {
			retval = RETVAL_ERROR;
		}
		else if(get_announcements(sockfd) < RETVAL_OK) {
			retval = RETVAL_ERROR;
		}
		else {
			/* If suck.restart exists something went wrong */
			if((fp = fopen(full_path(FP_GET, FP_TMPDIR, N_RESTART), "r" )) != NULL) {
				fscanf(fp,"%ld\n%d\n%d\n",&offset,&nron, &articleCount);
				fclose(fp);
				printf("Initiating restart at article %d\r\n",articleCount-nron);
  				retval = get_articles(offset, nron, articleCount, sockfd, MultiFile, status);
			}
			else if(get_message_index(&articleCount, sockfd) < RETVAL_OK) {
				retval = RETVAL_ERROR;
			}
			else if(articleCount == 0) {
 				printf("No articles.\r\n");
				retval = RETVAL_NOARTICLES;
			}
			else {
				unlink(full_path(FP_GET, FP_TMPDIR, N_SORTED));
				printf("Removing duplicates from list of %d articles.\r\n", articleCount);
				if((loop = do_dedupe()) >= 0) {
					articleCount = loop;
					retval = get_articles(offset, nron, articleCount, sockfd, MultiFile, status);
				}
			}	
			sputline( sockfd, "quit\r\n" );
		}
		if(sockfd >= 0) {
			close(sockfd);
			printf("Closed connection to %s\n",hi->h_name);
		}
		if(retval == RETVAL_OK) {
			switch(batch) {
			  case BATCH_INNXMIT:
				printf("Building Inn Batch File\n");
				do_innbatch(articleCount, batchfile);
				break;
			  case BATCH_RNEWS:
				printf("Building Rnews Batch File\n");
				do_rnewsbatch(articleCount, batchfile);
				break;
			  default:
				break;
			}
		}
	}	
	exit(retval);
}
/*--------------------------------------------------------------------*/
int get_message_index(int *articleCount, int sockfd) {

	int count,low,high,response,lastread,retval,i;
	char *sp,*inbuf,buf[MAXLINLEN],group[512];
	FILE *indexfp,*ifp,*tmpfp;

	retval = RETVAL_OK;
	indexfp = ifp = tmpfp = NULL;
	if((ifp = fopen(full_path(FP_GET, FP_DATADIR, N_OLDRC), "r" )) == NULL) {
		perror(full_path(FP_GET, FP_DATADIR, N_OLDRC));
		retval = RETVAL_ERROR;
	}
	else if((tmpfp = fopen(full_path(FP_GET, FP_TMPDIR, N_NEWRC), "w" )) == NULL) {
		perror(full_path(FP_GET, FP_TMPDIR, N_NEWRC));
		retval = RETVAL_ERROR;
	}
	else if((indexfp = fopen(full_path(FP_GET, FP_TMPDIR, N_INDEX), "w" )) == NULL) {
		perror(full_path(FP_GET, FP_TMPDIR, N_INDEX));
		retval = RETVAL_ERROR;
	}

	while(retval == RETVAL_OK && fscanf(ifp, "%s %d\n", group, &lastread) == 2) {
  		sprintf(buf,"group %s\r\n",group);
		sputline(sockfd,buf);
		i = sgetline(sockfd,&inbuf);
#ifdef DEBUG2
		do_debug("\ngroup %s\n%s",group,inbuf);
#endif
		if(i < 0) {
			retval = RETVAL_ERROR;
		}
		else if((sp = number(inbuf, &response)) == NULL) {
			fprintf(stderr,"Protocol error!\r\n");
			retval = RETVAL_ERROR;
		}
		else {
			if(response != 211) {
				fprintf(stderr,"GROUP <%s>, unexpected response %d\r\n", group, response );
   				continue; /* next newsgroup */
			}
			sp = number(sp, &count);
  			sp = number(sp, &low);
			sp = number(sp, &high);
  			fprintf(tmpfp, "%s %d\n", group, high);

			if (lastread >= 0 && lastread < high) {
   				if(lastread < low) {
					lastread = low - 1;
				}
   
   				sprintf(buf, "xhdr Message-ID %d-\r\n", lastread+1);
   				printf("%s...articles %d-%d\r\n", group, lastread+1, high);
   				sputline(sockfd, buf);
				i = sgetline(sockfd, &inbuf);
				number(inbuf, &response);
				if(i < 0) {
					retval = RETVAL_ERROR;
				}
				else if (response == 221) {
					do {
     						if(sgetline(sockfd, &inbuf) < 0) {
							retval = RETVAL_ERROR;
						}
						else if (*inbuf != '.' ) {
      							for(sp = inbuf; *sp != '<'; sp++) {
								 /* find begin of index */
								if (sp== inbuf+(MAXLINLEN-4)) {
									continue; /* the NNTP server tried to pull your leg */
								}
							}
      							fprintf(indexfp, "%s", sp);
      							*articleCount += 1;
						}
					} while (retval == RETVAL_OK && strcmp(inbuf, ".\n")!=0);
				} /* end if response */
   				else {
					fprintf(stderr,"Unexpected response %d getting IDs\r\n", response);
					retval = RETVAL_ERROR;
				}
			} /* end if lastread */
		} /* end else */
	} /* end while */

	if(indexfp != NULL) {
		fclose(indexfp);
	}
	if(tmpfp != NULL) {
		fclose(tmpfp);
	}
	if(ifp != NULL) {
		fclose(ifp);
	}
	return retval;
}
/*-----------------------------------------------------------*/
int get_articles(long offset, int nron, int articleCount, int sockfd, int MultiFile, int status_to_stdout) {

	int retval, logcount, mandatory;
	char buf[MAXLINLEN];
	FILE *logfp,*ifp, *sfp;

#ifdef KILLFILE
	PKillStruct killp = NULL;
#endif

	retval = RETVAL_OK;
	logfp = ifp = NULL;

	logcount = log10((double)articleCount) + 1;	/* how many digits wide is the articleCount */


#ifdef KILLFILE
	if((ifp = fopen(full_path(FP_GET, FP_DATADIR, N_KILLFILE), "r")) != NULL) {
		fclose(ifp);
		killp = parse_killfile();
	}
#endif		

	if((ifp = fopen(full_path(FP_GET, FP_TMPDIR, N_SORTED), "r" )) == NULL) {
		perror(full_path(FP_GET, FP_TMPDIR, N_SORTED));
		retval = RETVAL_ERROR;
	}
	else if(MultiFile == TRUE && checkdir(full_path(FP_GET, FP_MSGDIR, NULL)) == FALSE) {
		retval = RETVAL_ERROR;
	}
	else {
		fseek(ifp,offset,SEEK_SET); /* goto indexline */

	 	if((logfp = fopen(full_path(FP_GET, FP_TMPDIR, N_RESTART), "w" )) == NULL) {
			perror(full_path(FP_GET, FP_TMPDIR, N_RESTART));
			retval = RETVAL_ERROR;
		}
 		else {
			/* where do we print our status messages */
			sfp = (status_to_stdout == STATUS_STDERR) ? stderr : stdout; 
#ifdef TIMER
			TimerFunc(TIMER_START, 0);
#endif
			for(;retval == RETVAL_OK && fscanf(ifp, "%s %c", buf, (char *) &mandatory) == 2;) {
#ifdef DEBUG2
				do_debug("Msgnr = %s mandatory = %c\n", buf, mandatory);
#endif
				/* rewrite restart */
				rewind(logfp);
				fprintf(logfp,"%ld\n%d\n%d\n",offset,nron,articleCount);
  				fflush(logfp);
  				offset=ftell(ifp);
				nron++;
				if(MultiFile == TRUE || status_to_stdout == STATUS_STDERR)	{	
					/* if Multifile is false and we are printing to stdout don't display these messages */
#ifndef TIMER
  					fprintf(sfp, "%5d\r", articleCount - nron);
#else 
					fprintf(sfp, "%5d %s\r", articleCount - nron, TimerFunc(TIMER_DISPLAY, 0));
#endif
					fflush(sfp);	/* so message gets printed now */
				}
#ifdef KILLFILE
				if(mandatory == MANDATORY_YES || killp == NULL) {
					retval = get_one_article(sockfd, buf, nron, articleCount, logcount, MultiFile);
				}
				else {
					retval = get_one_article_kill(sockfd, buf, nron, articleCount, logcount, MultiFile, killp);
				}
#else
				retval = get_one_article(sockfd, buf, nron, articleCount, logcount, MultiFile);
#endif
 
		 	} /* end for */
			fclose(logfp);
			if(retval == RETVAL_OK && articleCount == nron) {
				unlink(full_path(FP_GET, FP_TMPDIR, N_RESTART));
			}
#ifdef TIMER
			if(retval == RETVAL_OK) {
				printf("%s\n", TimerFunc(TIMER_TOTALS, 0));
			}
#endif				
		}
		fclose(ifp);
	}
	return retval;
}
/*------------------------------------------------------------*/
int get_announcements(int sockfd) {
	char *inbuf;
	int retval = RETVAL_OK;
		
	/* Get the announcement line */
	if(sgetline(sockfd, &inbuf) < 0) {
		retval = RETVAL_ERROR;
	}
	else {
		printf("%s", inbuf );
#ifndef NNRP
		sputline(sockfd,"mode reader\r\n");	
		if( sgetline(sockfd,&inbuf) < 0) {	
			retval = RETVAL_ERROR;		  	
		}					
		else {					
			/* Again the announcement */
			printf("%s",inbuf);
		}
#endif
	}
	return retval;
}
/*--------------------------------------------------------------*/
/* read in each item fron index, and add it to link list and	*/
/* write out to sorted if it's not already there 		*/
/*--------------------------------------------------------------*/ 
int do_dedupe() {

	int retval, removed, kept, oldkept, oldremoved;
	FILE *fp;
	PList head, curr, prev; 
	char linein[MAXLINLEN+1];

	retval = RETVAL_OK;
	removed = kept = 0;		
	head = NULL;

	if((fp = fopen(full_path(FP_GET, FP_TMPDIR, N_INDEX), "r")) == NULL) {
		retval = RETVAL_ERROR;
		perror(full_path(FP_GET, FP_TMPDIR, N_INDEX));
	}
	else {
		while(retval == RETVAL_OK && fgets(linein, MAXLINLEN, fp) != NULL) {
			retval = build_list(linein, &head, &kept, &removed, MANDATORY_OPTIONAL);
		}
		fclose(fp);
	}
	if((fp = fopen(full_path(FP_GET, FP_DATADIR, N_SUPPLEMENTAL), "r")) != NULL) {
		oldkept = kept;
		oldremoved = removed;
		while(retval == RETVAL_OK && fgets(linein, MAXLINLEN, fp) != NULL) {
			if(linein[0] != '<') {
				fprintf(stderr, "Supplemental Invalid line: %s, ignoring\n", linein);
			}
			else {
				retval = build_list(linein, &head, &kept, &removed, MANDATORY_YES);
			}
		}
		printf("Supplement List Processed - %d messages added, %d messages removed\n", 
				kept-oldkept, removed-oldremoved); 
		fclose(fp);
	}
	if(head != NULL && kept > 0) {
		/* now write out whatever list we have*/
		if((fp = fopen(full_path(FP_GET, FP_TMPDIR, N_SORTED), "w")) == NULL) {
			retval = RETVAL_ERROR;
			perror(full_path(FP_GET, FP_TMPDIR, N_SORTED));
		}
		else {
			curr = head;
			while(curr != NULL && retval == RETVAL_OK) {
				if(fprintf(fp, "%s %c\n", curr->msgnr, curr->mandatory) == 0) {
					retval = RETVAL_ERROR;
					perror(full_path(FP_GET, FP_TMPDIR, N_SORTED));
				}
				curr = curr->next;
			}
			fclose(fp);
		}
	}
	/* all done, free up memory etc */
	curr = head;
	while(curr != NULL) {
		prev = curr->next;
		free(curr->msgnr);
		free(curr);
		curr = prev;
	}	
	if(retval == RETVAL_OK) {
		printf("Removed %d dupes, Downloading %d messages\n", removed, kept);
		retval = kept;
	}
	return retval;
}
/*--------------------------------------------------------------------------*/
int build_list(char *linein, PList *head, int *kept, int *removed, int mandatory) {

	PList curr, prev;
	int match, retval;

	retval = RETVAL_OK;
	linein[strlen(linein)-1] = '\0';	/* strip off NL */

	/* now see if it's in the linked list */
	curr = *head;
	if(curr == NULL) {
		/* special case, 1st node */
		if((*head = allocnode(linein, mandatory)) == NULL) {
			retval = RETVAL_ERROR;
		}
		else {
			(*kept)++;
		}
	}
	else {	
		do {
			match = strcmp(linein, curr->msgnr);
			prev = curr;
			curr = curr->next;			
		}
		while(match != 0 && curr != NULL);
		if(match == 0) {
			(*removed)++;
			prev->mandatory = (char) mandatory;			
			/* this is needed in case we are in mandatory list */
			/* and have a dupe of one in optional list */
		}
		else {
			/* reached end of list */
			if((prev->next = allocnode(linein, mandatory)) == NULL) {
				retval = RETVAL_ERROR;
			}
			else {
				(*kept)++;
			}
		}
	}
	return retval;
}
/*-----------------------------------------------------------------------*/
PList allocnode(char *linein, int mandatory) {
	
	PList ptr;

	if((ptr = malloc(sizeof(List))) == NULL) {
		fprintf(stderr, "Out of Memory, aborting");
	}
	else if((ptr->msgnr = calloc(strlen(linein)+1, sizeof(char))) == NULL) {
		fprintf(stderr, "Out of Memory, aborting");
		free(ptr);
	}
	else {
		strcpy(ptr->msgnr, linein);
		ptr->next = NULL;
		ptr->mandatory = (char) mandatory;
	}
	return ptr;
}
/*------------------------------------------------------------------------*/
/* check if directory exists, if not, try to create it.			  */
/* return TRUE if made/exists and can write to it			  */
/* -----------------------------------------------------------------------*/
int checkdir(char *dirname) {
	
	struct stat buf;
	int retval = TRUE;

	if(stat(dirname, &buf)== 0) {
		/* it exists */
		if(!S_ISDIR(buf.st_mode)) {
			fprintf(stderr, "%s: Not a directory\n", dirname);
			retval = FALSE;
		}
		else if(access(dirname, W_OK) != 0) {
			fprintf(stderr, "%s: Write permission denied\n", dirname);
			retval = FALSE;
		}
	}
	else if(errno == ENOENT) {
		/* doesn't exist, make it */
		if(mkdir(dirname, (S_IRUSR | S_IWUSR | S_IXUSR)) != 0) {
			perror(dirname);
			retval = FALSE;
		}
	}
	else {
		/* huh? */
		perror(dirname);
		retval = FALSE;
	} 
	return retval;	
}
/*---------------------------------------------------------*/
int do_innbatch(int articleCount, char *batchfile) {

	/* build batch file that contains article listing */
	/* needed by innxmit */
	int i, logcount, retval = RETVAL_OK;
	FILE *fptr;
	
#ifdef DEBUG2
	do_debug("BATCH FILE: %s\n", batchfile);
#endif
	if((fptr = fopen(batchfile, "w")) == NULL) {
		perror(batchfile);
		retval = RETVAL_ERROR;
	}
	else {
		logcount = log10((double) articleCount)+1;
		for(i=0;i<articleCount;i++) {
			/* full path puts the trailing slant on, so I don't have to here */
			fprintf(fptr, "%s%0*d-%d\n", full_path(FP_GET, FP_MSGDIR,""), logcount, i+1,articleCount);
		}
		fclose(fptr);
	}
	return retval;
}
/*----------------------------------------------------------*/
int do_rnewsbatch(int articleCount, char *batchfile) {
	/* build rnews formated file of msgs */
	
	int i, logcount, retval = RETVAL_OK;
	FILE *fpout, *fpin;
	char buf[MAXLINLEN], *tmp;
	struct stat sbuf;

#ifdef DEBUG2
	do_debug("BATCH FILE: %s\n", batchfile);
#endif
	if((fpout = fopen(batchfile, "w")) == NULL) {
		perror(batchfile);
		retval = RETVAL_ERROR;
	}
	else {
		logcount = log10((double) articleCount) +1 ;
		for(i=0;i<articleCount;i++) {
			sprintf(buf, "%0*d-%d", logcount, i+1,articleCount);
			tmp = full_path(FP_GET, FP_MSGDIR, buf);
			if(stat(tmp, &sbuf) == -1) {
				perror(tmp);
				retval = RETVAL_ERROR;
			}
			else {
				/* first put #! rnews size */
				fprintf(fpout, "#! rnews %ld\n", sbuf.st_size);
				if((fpin = fopen(tmp, "r")) == NULL) {
					perror(tmp);
					retval = RETVAL_ERROR;
				}
				else {
					while(fgets(buf, MAXLINLEN, fpin) != NULL) {
						fputs(buf, fpout);
					}
					fclose(fpin);
					unlink(tmp);	/* we are done with it, nuke it */
				}
			}
			
		}
		fclose(fpout);
	}


	return retval;
}
/*--------------------------------------------------------------*/
/* if which = FP_SET, store directory in array 			*/
/* 	    = FP_GET, retrieve directory and add file name 	*/
/* retval     FP_SET, if valid dir, the dir, else NULL		*/
/*	      FP_GET, full path					*/
/*--------------------------------------------------------------*/
char *full_path(int which, int dir, char *fname) {

	static char dirs[FP_NR_DIRS][_POSIX_PATH_MAX+1] = { N_TMPDIR, N_DATADIR, N_MSGDIR };
	static char path[_POSIX_PATH_MAX+1];	/* static so area valid after return */
	char *retptr;
	int ok;

	retptr = NULL;

	switch(which) { 
	  case FP_SET:
		ok = TRUE;
		switch(dir) {
		  case FP_TMPDIR:
			if(access(fname, W_OK) == -1) {
				perror(fname);
				ok = FALSE;
			}
			break;
		  case FP_DATADIR:
			if(access(fname, R_OK) == -1) {
				perror(fname);
				ok = FALSE;
			}
			break;
		  case FP_MSGDIR: /* do_nothing, this is checked elsewhere */
			break;
		}
		if(ok == FALSE) {
			retptr = NULL;
		}
		else {
			strncpy(dirs[dir], fname, sizeof(dirs[0]));
			retptr = dirs[dir];
		}
		break;
	  case FP_GET:
		/* put the right directory as the first part of the path */
		strcpy(path, dirs[dir]);
		/* if it contains a dit, then change to full path */
		if(path[0] == '.' && path[0] == '\0') {
			if(getcwd(path, PATH_MAX) == NULL) {
					perror(path);
					/* restore current path and pray */
					/* it will work later */
					strcpy(path, dirs[dir]);
			}
			else {
				/* tack on remainder of path */
				/* the 1 so skip . */
				strcat(path, &dirs[dir][1]);
			}		
		}
		if(fname != NULL) {
			/* if nothing to put on don't do any of this */
			/* put on trailing slant bar if needed */
			if(path[strlen(path)-1] != '/') {
				strcat(path, "/");
			}
			/* finally, put on filename */
			strcat(path, fname);
		}
		retptr = path;
		break;
	}
	return retptr;
}
/*-----------------------------------------------------------------------------------------*/
int get_one_article(int sockfd, char *msgnr, int nron, int articleCount, int logcount, int MultiFile) {

	char buf[MAXLINLEN+1];
	int retval, response;
	FILE *fptr;

	retval = RETVAL_OK;

	/* build command to get article header*/
	sprintf(buf, "article %s\r\n", msgnr);
	response = send_a_command(sockfd, buf, 220);

	if(response == RETVAL_ERROR) {
		retval = RETVAL_ERROR;
	}
	else if(response == RETVAL_OK) {
		if(MultiFile == TRUE) {
			/* open file */
			/* file name will be ####-#### ex 001-166 (nron,total) */
			sprintf(buf,"%0*d-%d", logcount, nron ,articleCount);
			if((fptr = fopen(full_path(FP_GET, FP_MSGDIR, buf), "w")) == NULL) {
				perror(full_path(FP_GET, FP_MSGDIR,buf));
				retval = RETVAL_ERROR;
			}
			else {
				retval = get_a_chunk(sockfd, fptr);
				fclose(fptr);
			}
		}
		else {
			retval = get_a_chunk(sockfd, stdout);
			fputs(".\n", stdout);	/* needed */
		}
	}
	return retval;
}
/*---------------------------------------------------------------------------*/
int get_a_chunk(int sockfd, FILE *fptr) {

	int done, partial, len, retval;
	char *inbuf;

	retval = RETVAL_OK;
	done = FALSE;
	partial = FALSE;

	/* partial = was the previous line a complete line or not */
	/* this is needed to avoid a scenario where the line is MAXLINLEN+1 */
	/* long and the last character is a ., which would make us think */
	/* that we are at the end of the message when we actually aren't */	
	while(done == FALSE) {
		len=sgetline(sockfd, &inbuf);
		if(len < 0) {
			retval = RETVAL_ERROR;
			done = TRUE;
		}
		else {
#ifdef TIMER
			TimerFunc(TIMER_ADDBYTES, len);
#endif								
			if(partial == FALSE && len == 2 && strcmp(inbuf, ".\n") == 0) {
				done = TRUE;
			}
			else {
				fputs(inbuf, fptr);
				fflush(fptr);
				partial= (len==MAXLINLEN&&inbuf[len-1]!='\n') ? TRUE : FALSE;
			}
		} 
	}
	return retval;
}
/*---------------------------------------------------------------------------*/
int send_a_command(int sockfd, char *cmd, int good_response) {

	char *inbuf;
	int response, retval;
#ifdef TIMER
	int len;
#endif
	retval = RETVAL_OK;

#ifdef DEBUG2
	do_debug("\nSENDING REQUEST\n%s",cmd);
#endif
 	sputline(sockfd,cmd);
#ifndef TIMER
 	if(sgetline(sockfd,&inbuf) < 0) {
#else
	if((len = sgetline(sockfd, &inbuf)) < 0) {
#endif
		retval = RETVAL_ERROR;
	}
	else {
#ifdef TIMER
		TimerFunc(TIMER_ADDBYTES, len);
#endif
		number(inbuf, &response);
		if (response != good_response) {
			fprintf(stderr,"*** Warning: Unexpected response\r\n*** %s", cmd);
			fputs(inbuf,stderr);
			retval = RETVAL_UNEXPECTEDANS;
		}
	}
	return retval;
}		
