#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <netdb.h>
#include <dirent.h>
#include <string.h>
#include <sys/stat.h>
#include <errno.h>
#include "both.h"

struct {
	char *command;
	int response;
} cmds[]  = { { "help\r\n", 100 }, { "list\r\n", 215 } };

enum { CMD_HELP, CMD_LIST };

/*-functions----*/
int do_a_command(int, int);

/*------------------------------------------------------------------*/
void main(int argc, char *argv[]) {

	int sockfd, response, cmd, retval = 0;
	struct hostent *hi;

	char *host, *buf, *ptr;
	
	/* allow testhost hosthame, testhost -a hostname */
	if(argc < 2 || argc > 3) { 
		retval = -1;
	}
	else {
		cmd = (argv[1][0] == '-' && argv[1][1] == 'a') ? CMD_LIST : CMD_HELP;
		if(cmd == CMD_LIST && argc == 2) {
			retval = -1;
		}
		host = (argc ==2) ? argv[1] : argv[2];
	} 	

	if(retval != 0) {
		fprintf(stderr, "Invalid command line\n");
		fprintf(stderr, "Usage: %s [-a] hostname\n", argv[0]);
		fprintf(stderr, "	-a = get active list\n");
	}
	else {
		sockfd = connect_to_nntphost( host, &hi);
		if(sockfd < 0 ) {
			retval = -1;
		}
		/* get the announcement line */
		else if(sgetline(sockfd, &buf) < 0) {
			retval = -1;
		}
		else {
			ptr = number(buf, &response);
			printf("%s",ptr);	/* print it */

			if(response != 200 && response != 201) {
				/* not a valid response */
				retval = -1;
			}
			else {
				do_a_command(sockfd, cmd);
			}
		}
		if(retval == -1) {
			fprintf(stderr, "Error talking to host %s\n", host);
		}
	}
	exit(retval);
}
/*-------------------------------------------------------------------------------*/
int do_a_command(int sockfd, int cmd) {
	char *ptr, *buf;
	int response;
	int len, done, retval = 0;

	sputline(sockfd,cmds[cmd].command);	/* which command do I run? */
	sgetline(sockfd, &buf);			/* get response */

	ptr = number(buf, &response);
	if(cmds[cmd].response != response) {
		fprintf(stderr, "%s\n", buf);
		retval = -1;
	}
	else {
		/* okay here we go */
		printf("%s", ptr); 
		/* commands are ended by a . on a line by itself */

		len = done = 0;
		while( len >=0 && done == 0) {
		
			len = sgetline(sockfd, &buf);

			if(len == 2 && strcmp(buf, ".\n") == 0) {
				done = 1;
			}
			else if(len > 0) {
				printf("%s",buf);
			}
		}
		if(len < 0) {
			retval = -1;
		}
	}
	return retval;
}
	
